#
#	Index Songs with Runtime in Milliseconds
#
#=======================================================

use strict;
use warnings;

use Cwd;
use Data::Dumper;
use Audio::Wav;
use File::Spec::Functions;
use File::Next;					
use File::Basename;
use Getopt::Long;

use constant{
	EXIFTOOL_FASTSCAN => 1,
	EXIFTOOL_VERBOSE => 0,
	EXIFTOOL_LARGEFILESUPPORT => 1,
	DEFAULT_FILE_SORT => 1,
	DEFAULT_FILE_RECURSION => 0,
};

my $result;
my $source_folder = getcwd;
my $source_file;
my $iteration;
my $filename;
my $base_name;
my $base_path;
my $base_ext;
my $duration_milliseconds;
my $opt_file_sort = DEFAULT_FILE_SORT;
my $opt_recursion = DEFAULT_FILE_RECURSION;
my $opt_no_sort = 0;
my @suffix_list = (".WAV");

$result = GetOptions (
			 "r|recurse" => \$opt_recursion,
		  	 "nosort" => \$opt_no_sort,
		         "f|source|path|directory|dir|folder=s" => \$source_folder,
			 "help|?" => sub{showHelp()},
		        );

if ($opt_no_sort) { 
	$opt_file_sort = 0;
}

$iteration = File::Next::files( { descend_filter => sub{$opt_recursion}, sort_files => $opt_file_sort}, $source_folder );


while ( defined ( $source_file = $iteration->() ) ) {
	($base_name,$base_path,$base_ext) = fileparse($source_file,@suffix_list);
	$filename = $base_name . $base_ext;
	if ($base_ext) {
		$duration_milliseconds = getSongLength($source_file);
		print "$filename: $duration_milliseconds\n";
	}
}

sub showHelp() {
	print "Usage:\n";
	print "recurse			Examine sub-folders (default: false)\n";
	print "source			Path to examine (default current)\n";
	exit();	
}

#-----------------------------------------------------------
# Get Song Length
#-----------------------------------------------------------
sub getSongLength{
	my $pathname = shift;

	my $song_length = 0;

   	my $wav = new Audio::Wav;
    	my $read = $wav -> read($pathname);
    	my $details = $read -> details();
	my $audio_samples = $read -> length_samples();
	my $sample_rate = $details->{'sample_rate'};

	$song_length = int (($audio_samples / $sample_rate) * 1000);

	return $song_length;
}

#--------------------------------------------------
# Trim String
#--------------------------------------------------
sub trimString {
	my $tempvar = shift;

	$tempvar = "" unless defined $tempvar;

    	$tempvar =~ s/^\s+//;
    	$tempvar =~ s/\s+$//;

	return $tempvar;
}

#--------------------------------------------------
# Remove Substring
#--------------------------------------------------
sub removeSubstring {
	my $content = shift;
	my $match_string = shift;

	return $content unless defined $content;
	return $content unless defined $match_string;

	$match_string = quotemeta($match_string);

	$content =~ s/$match_string//g;

	return $content;
}

